var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
/**
 * 調用Worker中的方法。
 *
 * @param funcName 方法名稱。
 * @params params 方法參數。
 * @param workerName 是否在Worker中執行，默認為true。。
 * @returns
 */
export function callWorkerFunction(funcName, params, workerName) {
    return __awaiter(this, void 0, void 0, function () {
        var _result, start, _a;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    _result = {};
                    if (!workerFunctions.has(funcName)) {
                        console.warn("\u672A\u8A3B\u518AWorkerFunction ".concat(funcName, " \uFF0C\u8ACB\u4F7F\u7528\u65B9\u6CD5 registerWorkerFunction \u8A3B\u518AWorkerFunction\u3002"));
                        return [2 /*return*/, _result];
                    }
                    start = Date.now();
                    _a = _result;
                    return [4 /*yield*/, runWorkerFunction(workerName, funcName, params)];
                case 1:
                    _a.result = _b.sent();
                    _result.cost = Date.now() - start;
                    return [2 /*return*/, _result];
            }
        });
    });
}
function getWorker(workerName) {
    return workers.get(workerName);
}
function getFunction(funcName) {
    return workerFunctions.get(funcName);
}
function runWorkerFunction(workerName, funcName, params) {
    return __awaiter(this, void 0, void 0, function () {
        var worker, promise, func, result;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    worker = getWorker(workerName);
                    if (worker) {
                        promise = new Promise(function (resolve) {
                            worker.onmessage = function (_a) {
                                var res = _a.data;
                                resolve(res);
                            };
                        });
                        worker.postMessage({ funcName: funcName, params: params });
                        return [2 /*return*/, promise];
                    }
                    if (workerName) {
                        console.warn("\u672A\u627E\u5230\u540D\u79F0\u4E3A ".concat(workerName, " \u7684Worker\uFF0C\u8BF7\u4F7F\u7528registerWorker\u8A3B\u518AWorker\uFF01\u5F3A\u5236\u8FD0\u884C\u5728\u5F53\u524DWorker\u4E2D\u3002"));
                    }
                    func = getFunction(funcName);
                    return [4 /*yield*/, func.apply(undefined, params)];
                case 1:
                    result = _a.sent();
                    return [2 /*return*/, result];
            }
        });
    });
}
/**
 * 註冊可在Worker中執行的方法。
 *
 * @param funcName 方法名稱。
 * @param func 方法定義。
 *
 */
export function registerWorkerFunction(funcName, func) {
    if (workerFunctions.has(funcName)) {
        console.warn("\u91CD\u8907\u8A3B\u518AWorkerFunction ".concat(funcName, " "), func, workerFunctions.get(funcName));
        return;
    }
    workerFunctions.set(funcName, func);
}
/**
 * 註冊Worker。
 *
 * @param workerName Worker名稱。
 * @param worker Worker實例。
 *
 */
export function registerWorker(workerName, worker) {
    if (workers.has(workerName)) {
        console.warn("\u91CD\u8907\u8A3B\u518AWorker ".concat(workerName, " "), worker, workers.get(workerName));
        return;
    }
    workers.set(workerName, worker);
}
/**
 * 注销Worker。
 *
 * @param workerName Worker名稱。
 * @param terminate 是否終止Worker。
 */
export function ungisterWorker(workerName, terminate) {
    if (terminate === void 0) { terminate = true; }
    if (workers.has(workerName)) {
        var worker = workers.get(workerName);
        workers.delete(workerName);
        if (terminate) {
            worker.terminate();
        }
    }
}
var workerFunctions = new Map();
var workers = new Map();
//# sourceMappingURL=WorkerFunction.js.map